@@ -2,7 +2,7 @@ if (!library(Rmpi,logical.return = TRUE)){
     warning("Rmpi cannot be loaded")
     q(save = "no")
 }
-options(error=quote(assign(".mpi.err", TRUE, env = .GlobalEnv)))
+options(error=quote(assign(".mpi.err", TRUE, envir = .GlobalEnv)))
 .comm <- 1
 .intercomm <- 2
 invisible(mpi.comm.get.parent(.intercomm))
@@ -10,8 +10,10 @@ invisible(mpi.intercomm.merge(.intercomm,1,.comm))
 invisible(mpi.comm.set.errhandler(.comm))
 mpi.hostinfo(.comm)
 invisible(mpi.comm.disconnect(.intercomm))
+nonblock <- as.logical(mpi.bcast(integer(1),type=1,rank=0,comm=.comm))
+sleep <- mpi.bcast(double(1),type=2,rank=0,comm=.comm)
 repeat 
-    try(eval(mpi.bcast.cmd(rank=0,comm=.comm),envir=sys.parent()),TRUE)
+    try(eval(mpi.bcast.cmd(rank=0,comm=.comm, nonblock=nonblock, sleep=sleep),envir=sys.parent()),TRUE)
 print("Done")
 invisible(mpi.comm.disconnect(.comm))
 invisible(mpi.comm.set.errhandler(0))
